package fsplParser

import "io"
import "testing"
import "strings"
import "git.tebibyte.media/fspl/fspl/lexer"
import "git.tebibyte.media/fspl/fspl/errors"
import "git.tebibyte.media/fspl/fspl/testcommon"

func testString (test *testing.T, correct string, input string) {
	testStringInternal(test, correct, input, false)
}

func testStringSkim (test *testing.T, correct string, input string) {
	testStringInternal(test, correct, input, true)
}

func testStringInternal (test *testing.T, correct string, input string, skim bool) {
	ast := Tree { }
	lx, err := lexer.LexReader("input.fspl", strings.NewReader(input))
	if err != nil && err != io.EOF{
		test.Error("lexer returned error:\n" + errors.Format(err))
		return
	}

	if skim {
		err = ast.Skim(lx)
	} else {
		err = ast.Parse(lx)
	}
	if err != nil && err != io.EOF{
		test.Error("parser returned error:\n" + errors.Format(err))
		return
	}

	got := ast.String()
	if got != correct {
		test.Logf("results do not match")
		testcommon.Compare(test, correct, got)
		test.Log("SOURCE FSPL CODE:")
		test.Log("\033[32m" + input + "\033[0m")
		test.Fail()
	}
}
